
/* Virtuino Bluetooth example 4 without library (Minimum code: Only the virtual V memory is supported)
 * All the necessary code is included in this sketch
 * Supported boards: MEGA, DUE and any other board with Hardware Serial connection
 * Example name = "Bluetooth example via hardware serial port- Only the V memory is supported in this code"
 * Created by Ilias Lamprou
 * Updated Jun 09 2019
 * This is the minimum code
 * Download latest Virtuino android app from the link: https://play.google.com/store/apps/details?id=com.virtuino_automations.virtuino
 */

// Code to use Serial1 port  pins 19,18
// connect the arduino RX pin 19 to bluetooth module TX pin   
//connect the arduino TX pin 18 to bluetooth module RX pin.  

String inputBuffer = "";        // a String to store the incoming data

#define V_memorySize 32         // Virtuino V memory size (default is 32). You can increase it to 99 or to decrease it in case you want to reduce the memory
float V_memory[V_memorySize];   // create an array to store the V memory variables. This array will synchronized with Virtuino V memory

unsigned long t=0;
int oldState=-1;
//=================================================== setup
//===================================================
void setup() {
  // initialize serial:
  Serial.begin(9600);
  Serial1.begin(9600);    // This has to be the same with the bluetooth module serial baud rate
  
  inputBuffer.reserve(200); // reserve 200 bytes for the inputBuffer:
  pinMode(12,OUTPUT);       // On Virtuino panel add a switch to control this pin. Select the pin V12
  pinMode(13,OUTPUT);       // On Virtuino panel add a switch to control this pin. Select the pin V13
  
  pinMode(6,INPUT);         // Connect a switch to this pin
  vMemoryWrite(5,23.45);    // Example: how to write an initial value to V memory 5. On Virtuino add a value display to V5 to read this value
}

//=================================================== setup
//===================================================
void loop() {
  virtuinoRun();    // serial handler

  //-- example on how to send sensor values every 3 seconds to Virtuino
  if (millis()-t>3000) {    // every 3 seconds send the sensors' values to the Virtuino app
      vMemoryWrite(0,random(100));     // replace with your sensor variable. Write the sensor value to V memory 0
      vMemoryWrite(1,random(100));    // replace with your sensor variable. Write the sensor value to V memory 1
      t=millis();
  }

 //-- example on how to read a variable from Virtuino
  if (oldState!=vMemoryRead(6)) {   // check if the state of the V6 has changed
    if (vMemoryRead(6)==1){ 
     // do something  
     Serial.println("V6=1");  // check the value of a Virtual pin
     sendCommand('M',0, "I love Virtuino");   // reply with a message
    }
    else {Serial.println("V6=0");}// do something else 
    oldState=vMemoryRead(6);   // store the last state
  }

  //-- example on how to enable or disable a pin using the V memory variable
  if (vmemoryRead(12)==1) digitalWrite(12,HIGH); else digitalWrite(12,LOW);   // The V12 virtuino pin controls the digital pin 12
  if (vmemoryRead(13)==1) digitalWrite(13,HIGH); else digitalWrite(13,LOW);   // The V13 virtuino pin controls the digital pin 13
  //example: if (vmemoryRead(pin)==1) digitalWrite(pin,HIGH); else digitalWrite(pin,LOW);

  
  vDelay(1000);    // use this vDelay instead of the default delay()
}





//=================================================== onCommandReceived
//===================================================
/*This void is called every time the Virtuino app wants to enable or disable pin or to write values to variables
 */
void onCommandReceived(char type, int pin, String data){
  Serial.println("/nonCommandReceived: "+String(type)+" "+String(pin)+" "+data);
  switch (type){
    case 'V' :     // type: V memory  
         if ((pin>=0) & (pin<V_memorySize)) V_memory[pin]=data.toFloat();   // V0
    break;
     case 'C' : onConnected();
                data="My firmware code";   // Send a firmware code to the Virtuino app
      break;
    
  }
  sendCommand(type,pin,data);    // If the 'confirmation' option on the server settings is enable the code has to send a reply with the same command 
}

//=================================================== onConnected
//===================================================
//This void is called every time the Virtuino app is connected with the bluetooth module
void onConnected(){
  Serial.println("Connected");
  sendCommand('M',0, "Hello Virtuino");     // Send a message to the Virtuino app
                                            // The app will show the text as Toast message
  //sendCommand('V',2, 128);                // You can send from here some variables as starting values
}

//=================================================== onCommandReceivedForInfo
//===================================================
/*This void is called every time the Virtuino app wants to read the state of the digital IOs,
  variables, sensors, analog inputs, PWM pins
  You have to send a replay with the correct value or nothing
  V = V memory (float)
  D = DV memory (int)
  Q = Input or output state (0 or 1)
  A = Analog input value 
  O = Analog output value 
  T = text value
  C = on connected state
  */

void onCommandReceivedForInfo(char type, int pin){
  Serial.println("\nonCommandReceivedForInfo type="+String(type)+"  pin="+String(pin));
  if (type=='V'){
        if ((pin>=0) & (pin<V_memorySize)) sendCommand('V',pin,String(V_memory[pin]));   // V1
  }
 }

//=================================================== sendCommand
/* Call this void every time you want to send data to Virtuino app
 * Avoid to call this void on every cycle of void loop. 
 * Type can be a char like V, D, Q, A, O
 * V = V memory (float)
 * D = DV memory (int)
 * Q = Input or output state (0 or 1)
 * O = Analog output value (0-255)
 */
void sendCommand(char type, int pin, String data){
  Serial1.print('!');
  Serial1.print(type);
  if (pin<10) Serial1.print('0');
  Serial1.print(String(pin));
  Serial1.print('=');
  Serial1.print(data);
  Serial1.print('$');
}

//=================================================== virtuinoRun
/*
  SerialEvent occurs whenever a new data comes in the hardware serial RX. This
  routine is run on every cycle of void loop, so using delay inside loop can
  delay response. 
*/
void virtuinoRun(){
  while (Serial1.available()) {
    char inChar = (char)Serial1.read();
    Serial.print(inChar);
    if (inChar=='!') inputBuffer = "";
    else if (inChar=='$') {
      char type=inputBuffer.charAt(0);
      if (inputBuffer.charAt(3)=='='){
        int pin = inputBuffer.substring(1,3).toInt();
        if (inputBuffer.charAt(4)=='?') {
          onCommandReceivedForInfo(type,pin);
        }
        else onCommandReceived(type,pin,inputBuffer.substring(4));
      }
      inputBuffer = "";
      break;
    }
    else inputBuffer += inChar;
  }
}
//====================================================== vDelay
/* Use this void instead of the default Delay 
 * This void doesn't block the communication 
 */
void vDelay(int milliseconds){
   unsigned long m=millis();
   while (millis()-m<milliseconds) virtuinoRun();
}

//=====================================================
void vMemoryWrite(int pin,float value){
  if ((pin>=0) & (pin<V_memorySize)) V_memory[pin]=value; 
}
//=====================================================
float vMemoryRead(int pin){
  if ((pin>=0) & (pin<V_memorySize)) return V_memory[pin]; 
  else return 0;
}


